/**
 * Copyright (c) 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */

/* eslint-disable no-console */
/* eslint-disable import/no-unresolved */

process.env.NODE_ENV = 'production';

const path = require('path');
const fs = require('fs-extra');
const webpack = require('webpack');
const packageJson = require('../../../package.json');
const config = require('../config/webpack.config.prod');

const compiler = webpack(config);
const distPath = config.output.path;

// Removing existing `dist` directory...
fs.remove(distPath)
  // Running webpack compiler...
  .then(() => new Promise((resolve, reject) => {
    console.log('Compiling...');
    compiler.run((error, stats) => (error ? reject(error) : resolve(stats)));
  }))
  // Displaying webpack compilation stats...
  .then((stats) => {
    if (stats.hasErrors()) throw new Error(stats.toJson().errors[0]);
    console.log(stats.toString({
      colors: true,
      cached: true,
      performance: true,
      modules: false,
      children: false,
    }));
  })
  // Writing distributable `package.json` file into `dist` directory...
  .then(() => {
    if (config.target !== 'web') {
      // `package.json` content used for distribution.
      const distPackageJson = JSON.stringify({
        name: packageJson.name,
        main: `./${packageJson.name}.js`,
        bugs: packageJson.bugs,
        author: packageJson.author,
        version: packageJson.version,
        engines: packageJson.engines,
        licence: packageJson.licence,
        keywords: packageJson.keywords,
        homepage: packageJson.homepage,
        repository: packageJson.repository,
        description: packageJson.description,
        contributors: packageJson.contributors,
        dependencies: packageJson.dependencies,
      });

      return fs.writeFile(path.join(distPath, 'package.json'), distPackageJson);
    }
    return null;
  })
  // All went well...
  .then(() => {
    console.log('Done.');
  })
  // If any error occurs...
  .catch((error) => {
    console.error(error);
  });